##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

""" MySQL Database Documentation object
Documentation of MySQL Databases.
"""

__version__ = '$Revision: 2.0 $'[11:-2]

import Globals, AccessControl, string, MySQLTableDocumentation
from Globals import DTMLFile, ImageFile
from OFS import Folder
from AccessControl import getSecurityManager, ClassSecurityInfo


manage_addMySQLDBDocumentationForm = DTMLFile( 'dtml/addMySQLDBDocumentationForm', globals() )


def manage_addMySQLDBDocumentation( self, conn, id='', title='', desc='', createUserF=0, REQUEST=None ):
   """
   Method called by manage_addMySQLDBDocumentationForm.
   It instanciates the class MySQLDBDocumentation.
   """
   if (id == ''):
      conn_obj = getattr( self, conn )
      id = string.split( conn_obj.connection_string, '@' )[0]
   ob = MySQLDBDocumentation()
   ob.id = id
   ob.title = title
   ob.desc = desc
   ob.conn = conn
   self._setObject( id, ob )
   ob = self._getOb( id )
   
   checkPermission = getSecurityManager().checkPermission
   if not checkPermission( 'Add Documents, Images, and Files', ob ):
      raise 'Unauthorized', ( 'You are not authorized to add DTML Documents.' )
   ob.manage_addDTMLDocument( 'index_html', title, DTMLFile( 'dtml/default_index', globals() ) )

   if createUserF:
      if not checkPermission( 'Add User Folders', ob ):
	 raise 'Unauthorized', ( 'You are not authorized to add User Folders.' )
      ob.manage_addUserFolder()

   if REQUEST is not None:
      return self.manage_main( self, REQUEST )

      
class MySQLDBDocumentation( Folder.Folder ):
   """
   Base Classe for MySQL Database Documentation Product
   """

   meta_type = 'MySQL Database Documentation'

   _properties=({ 'id' : 'title', 'type' : 'string' }, 
   		{ 'id' : 'desc', 'type' : 'text' }, 
   		{ 'id' : 'conn', 'type' : 'string' } 
	       )

   manage_options = (
      Folder.Folder.manage_options +
      (
	 { 'label' : 'Upload diagram', 'action' : 'manage_uploadDiagramForm', 'help' : ( 'MySQLDBDocumentation', 'MySQL-Database-Documentation_Upload_diagram.stx' ) },
      )
   )
   

   meta_types = ( 'MySQL Table Documentation' )

   manage_addMySQLTableDocumentationForm = DTMLFile( 'dtml/addMySQLTableDocumentationForm', globals() )
   manage_uploadDiagramForm = DTMLFile( 'dtml/uploadDiagramForm', globals() )
   manage_addMySQLTableDocumentation = MySQLTableDocumentation.manage_addMySQLTableDocumentation
   manage_main = DTMLFile( 'dtml/db_main', globals() )

   security = ClassSecurityInfo()

   def connection_string( self ):
      """Return the first part (db_name@host_name)
      of the connection string of the associated MySQL Database Connection object.
      """
      try:
         conn_obj = getattr( self, self.conn )
	 return (string.split( conn_obj.connection_string, ' ' ))[0]
      except:
	 return ''
	 
         
   def db_name( self ):
      """Return the associated database name.
      """
      try:
         conn_obj = getattr( self, self.conn )
	 return (string.split( conn_obj.connection_string, '@' ))[0]
      except:
         return ''
	 
      
   def connected( self ):
      """Check if the associated MySQL Database Connection object
      still exists and is still connected.
      """
      try:
         conn_obj = getattr( self, self.conn )
	 return conn_obj.connected
      except:
         return 0


   def tables( self ):
      """Return the tables list of the
      associated database.
      """
      try:
         tables = []
         conn_obj = getattr( self, self.conn )
         db_tmpl = conn_obj.factory()
         for i in db_tmpl(conn_obj.connection_string).tables():
            tables.append( i['TABLE_NAME'] )
         return tables
      except:
         return []


   def documented_tables( self ):
      """Return the tables of the associated database
      that are documented in the current db documentation.
      """
      return self.objectIds( 'MySQL Table Documentation' )


   def undocumented_tables( self ):
      """Returned the tables of the associated database
      that are not documented in the current db documentation.
      """
      undoc_tables = []
      doc_tables = self.documented_tables()
      for i in self.tables():
	 if i not in doc_tables:
	    undoc_tables.append( i )
      return undoc_tables

   
   def deleted_tables( self ):
      """Returned the tables that are documented in the
      current db documentation and that have been deleted from
      the associated database.
      """
      del_tables = []
      tables = self.tables()
      for i in self.documented_tables():
	 if i not in tables:
	    del_tables.append( i )
      return del_tables

      
   def column_names( self, table ):
      """Return the column names of the given table.
      """
      try:
         columns = []
         conn_obj = getattr( self, self.conn )
	 db_tmpl = conn_obj.factory()
	 for i in db_tmpl(conn_obj.connection_string).columns( table ):
	    columns.append( i['Name'] )
	 return columns
      except:
         return []

   
   def column_props( self, table ):
      """Return the columns properties of the given table.
      """
      try:
         conn_obj = getattr( self, self.conn )
         db_tmpl = conn_obj.factory()
         return db_tmpl(conn_obj.connection_string).columns( table )
      except:
         return []


   def documented_columns( self, table ):
      """Return the columns of the given table
      that are documented in the current documentation.
      """
      columns = self.column_names( table )
      doc_columns = getattr( self, table ).propertyIds()
      really_doc_columns = []
      for i in columns:
	 if i in doc_columns:
	    really_doc_columns.append( i )
      return really_doc_columns

      
   def undocumented_columns( self, table ):
      """Return the columns of the given table
      that are not documented in the current documentation.
      """
      columns = self.column_names( table )
      doc_columns = getattr( self, table ).propertyIds()
      undoc_columns = []
      for i in columns:
	 if i not in doc_columns:
	    undoc_columns.append( i )
      return undoc_columns
	 
	 
   def deleted_columns( self, table ):
      """Return the documented columns that
      have been deleted or renamed in the database.
      """
      columns = self.column_names( table )
      doc_columns = getattr( self, table ).propertyIds()
      deleted_columns = []
      for i in doc_columns:
	 if ((i != 'title') and (i != 'desc') and (i != 'order')):
	    if i not in columns:
	       deleted_columns.append( i )
      return deleted_columns

     
   security.declareProtected( 'Add MySQL Table Documentations', 'manage_autoDocumentate' )
   def manage_autoDocumentate( self, REQUEST=None ):
      """Add as many MySQL Table Documentation objects
      as there are undocumented tables.
      """
      cur_order = len(self.objectIds( 'MySQL Table Documentation' ))
      for i in self.undocumented_tables():
	 self.manage_addMySQLTableDocumentation( i, cur_order )
	 cur_order = cur_order + 1

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )
	 

   security.declareProtected( 'Change MySQL Table Documentations', 'manage_upMySQLTableDocumentation' )
   def manage_upMySQLTableDocumentation( self, ids=[], REQUEST=None ):
      """Place the selected MySQL Table Documentation object
      before the previous one.
      """
      if (ids != []):
	 cur_tdoc = getattr( self, ids[0] )
	 cur_order = cur_tdoc.order
	 if (cur_order > 0):
	    # Change the order of previous table doc
	    for i in self.objectValues( 'MySQL Table Documentation' ):
	       if (i.order == cur_order-1):
	          i.order = cur_order
	          break
	    # Change the order of the selected table doc
	    cur_tdoc.order = cur_order - 1

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )
	 

   security.declareProtected( 'Change MySQL Table Documentations', 'manage_downMySQLTableDocumentation' )
   def manage_downMySQLTableDocumentation( self, ids=[], REQUEST=None ):
      """Place the selected MySQL Table Documentation object
      after the next one.
      """
      if (ids != []):
	 cur_tdoc = getattr( self, ids[0] )
	 cur_order = cur_tdoc.order
	 docs = self.objectValues( 'MySQL Table Documentation' )
	 if (cur_order < len(docs)-1):
            # Change the order of previous table doc
	    for i in docs:
	       if (i.order == cur_order+1):
		  i.order = cur_order
		  break
            # Change the order of the selected table doc
	    cur_tdoc.order = cur_order + 1

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )


   security.declareProtected( 'Change MySQL Table Documentations', 'manage_delMySQLTableDocumentations' )
   def manage_delMySQLTableDocumentations( self, ids=[], REQUEST=None ):
      """Delete the selected MySQL Table Documentation objects.
      """
      docs = self.objectValues( 'MySQL Table Documentation' )
      for i in ids:
	 obj = getattr( self, i )
	 cur_order = obj.order
	 for doc in docs:
	    if (doc.order > cur_order):
	       doc.order = doc.order - 1
         self.manage_delObjects( [i] )

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )
	 
	
   def manage_uploadDiagram( self, file, REQUEST ):
      """Add or replace the Image object associated
      to the current MySQL Database Documentation object
      that represent the diagram of the database.
      """
      db_name = self.db_name()
      if (hasattr( self, db_name + '_diagram' )):
	 getattr( self, db_name + '_diagram' ).manage_upload( file )
      else:
         self.manage_addImage( id=(db_name + '_diagram'), title=(db_name + ' diagram'), file=file )

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )
	 
	    
