##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

import Globals, Persistence, Acquisition, AccessControl
from Globals import DTMLFile, ImageFile
from OFS import SimpleItem, PropertyManager
from Persistence import Persistent
from AccessControl import ClassSecurityInfo


def manage_addMySQLTableDocumentation( self, id, order=-1, title='', desc='', REQUEST=None ):
   """
   Method called by manage_addMySQLTableDocumentationForm.
   It instanciates the class MySQLTableDocumentation.
   """
   if (order < 0):
      order = len( self.objectValues( 'MySQL Table Documentation' ) )
   ob = MySQLTableDocumentation()
   ob.id = id
   ob.title = title
   ob.desc = desc
   ob.order = order
   self._setObject( id, ob )

   if REQUEST is not None:
      return self.manage_main( self, REQUEST )

      
class MySQLTableDocumentation( PropertyManager.PropertyManager,
			   Acquisition.Implicit,
			   Persistence.Persistent,
	                   SimpleItem.Item ):
   """
   MySQL Table Documentations objects are contained 
   only by MySQL Database Documentations objects.
   """

   meta_type = 'MySQL Table Documentation'

   _properties=({ 'id' : 'title', 'type' : 'string' }, { 'id' : 'desc', 'type' : 'text' }, 
   		{ 'id' : 'order', 'type' : 'int' } )

   manage_options = (
     (
        { 'label' : 'Report', 'action' : 'manage_main', 'help' : ( 'MySQLDBDocumentation', 'MySQL-Table-Documentation_Report.stx' ) },
	{ 'label' : 'Properties', 'action' : 'manage_propertiesForm', 'help' : ( 'MySQLDBDocumentation', 'MySQL-Table-Documentation_Properties.stx' ) }, 
     ) +
     SimpleItem.Item.manage_options
   )

   meta_types = ()

   methods = {}

   __ac_permissions__ = ()

   misc_ = {
      'MySQLTableDocumentationIcon' : ImageFile( 'icons/MySQLTableDocumentation.gif', globals() ),
   }

   manage_main = DTMLFile( 'dtml/table_main', globals() )
   manage_propertiesForm = DTMLFile( 'dtml/properties', globals() )

   security = ClassSecurityInfo()

   
   security.declareProtected( 'Change MySQL Table Documentations', 'manage_repare' )
   def manage_repare( self, REQUEST=None ):
      """Delete the documented columns of the current
      MySQL Table Documentation object that have been
      removed or renamed in the database.
      """
      del_cols = self.aq_parent.deleted_columns( self.id )
      self.manage_delProperties( del_cols )

      if REQUEST is not None:
         return self.manage_main( self, REQUEST )


   security.declareProtected( 'Change MySQL Table Documentations', 'manage_addMySQLFieldDocumentation' )
   def manage_addMySQLFieldDocumentation( self, id, desc='', REQUEST=None ):
      """Documentate the selected column of the current table.
      """
      self.manage_addProperty( id=id, type='string', value=desc )

      if REQUEST is not None:
	 return self.manage_main( self, REQUEST )
      

      
      
